@@ -2,13 +2,14 @@
 # create a base image, convert to btrfs, remove all files, rollback the ext4 image
 # note: ext4 only
 
-source "$TEST_TOP/common"
-source "$TEST_TOP/common.convert"
+source "$TEST_TOP/common" || exit
+source "$TEST_TOP/common.convert" || exit
+
+check_prereq btrfs-convert
+check_global_prereq mke2fs
 
 setup_root_helper
 prepare_test_dev
-check_prereq btrfs-convert
-check_global_prereq mke2fs
 
 # simple wrapper for a convert test
 # $1: btrfs features, argument to -O
@@ -60,7 +61,9 @@ do_test() {
 	rm "$CHECKSUMTMP"
 }
 
-for feature in '' 'extref' 'skinny-metadata' 'no-holes'; do
+# Iterate over defaults and options that are not tied to hardware capabilities
+# or number of devices
+for feature in '' 'block-group-tree' ; do
 	do_test "$feature" "ext4 4k nodesize" 4096 mke2fs -t ext4 -b 4096
 	do_test "$feature" "ext4 8k nodesize" 8192 mke2fs -t ext4 -b 4096
 	do_test "$feature" "ext4 16k nodesize" 16384 mke2fs -t ext4 -b 4096
